45#define OFFSET(x) offsetof(DBlurContext, x)
46#define FLAGS AV_OPT_FLAG_VIDEO_PARAM|AV_OPT_FLAG_FILTERING_PARAM|AV_OPT_FLAG_RUNTIME_PARAM
57#define f(n, m) (dst[(n) * width + (m)])
62 const float b0 =
s->b0;
63 const float b1 =
s->b1;
66 float *
dst =
s->buffer;
70 for (
int y = 1; y <
height; y++) {
71 g = q *
f(y, 0) +
c *
f(y, 0);
72 for (
int x = 0; x <
width; x++) {
73 f(y, x) =
b0 *
f(y, x) +
b1 *
f(y - 1, x) +
g;
74 g = q *
f(y, x) +
c *
f(y - 1, x);
78 for (
int y =
height - 2; y >= 0; y--) {
80 for (
int x =
width - 1; x >= 0; x--) {
81 f(y, x) =
b0 *
f(y, x) +
b1 *
f(y + 1, x) +
g;
82 g = q *
f(y, x) +
c *
f(y + 1, x);
86 for (
int y = 1; y <
height; y++) {
88 for (
int x =
width - 1; x >= 0; x--) {
89 f(y, x) =
b0 *
f(y, x) +
b1 *
f(y - 1, x) +
g;
90 g = q *
f(y, x) +
c *
f(y - 1, x);
94 for (
int y =
height - 2; y >= 0; y--) {
95 g = q *
f(y, 0) +
c *
f(y, 0);
96 for (
int x = 0; x <
width; x++) {
97 f(y, x) =
b0 *
f(y, x) +
b1 *
f(y + 1, x) +
g;
98 g = q *
f(y, x) +
c *
f(y + 1, x);
109 const int width =
s->planewidth[plane];
110 const int height =
s->planeheight[plane];
152 s->depth =
desc->comp[0].depth;
154 s->planewidth[0] =
s->planewidth[3] = inlink->
w;
156 s->planeheight[0] =
s->planeheight[3] = inlink->
h;
169 float mu, nu,
R1,
R2, w1, w2;
172 angle = angle *
M_PI / 180.f;
176 R1 = (mu *
r) * (mu *
r);
177 R2 = (nu *
r) * (nu *
r);
178 s->R3 = mu * nu *
r *
r;
181 a0 = (w1 + 0.5f) * (w2 + 0.5f) -
fabsf(
s->R3);
212 for (plane = 0; plane <
s->nb_planes; plane++) {
213 const int height =
s->planeheight[plane];
214 const int width =
s->planewidth[plane];
215 float *bptr =
s->buffer;
216 const uint8_t *
src = in->
data[plane];
217 const uint16_t *src16 = (
const uint16_t *)in->
data[plane];
218 const float *src32 = (
const float *)in->
data[plane];
219 uint8_t *
dst =
out->data[plane];
220 uint16_t *dst16 = (uint16_t *)
out->data[plane];
221 float *dst32 = (
float *)
out->data[plane];
224 if (!(
s->planes & (1 << plane))) {
233 for (y = 0; y <
height; y++) {
234 for (x = 0; x <
width; x++) {
240 }
else if (
s->depth <= 16) {
241 for (y = 0; y <
height; y++) {
242 for (x = 0; x <
width; x++) {
249 for (y = 0; y <
height; y++) {
250 for (x = 0; x <
width; x++) {
251 memcpy(bptr, src32,
width *
sizeof(
float));
262 for (y = 0; y <
height; y++) {
263 for (x = 0; x <
width; x++) {
267 dst +=
out->linesize[plane];
269 }
else if (
s->depth <= 16) {
270 for (y = 0; y <
height; y++) {
271 for (x = 0; x <
width; x++) {
275 dst16 +=
out->linesize[plane] / 2;
278 for (y = 0; y <
height; y++) {
279 for (x = 0; x <
width; x++) {
280 memcpy(dst32, bptr,
width *
sizeof(
float));
283 dst32 +=
out->linesize[plane] / 4;
305 .p.priv_class = &dblur_class,
uint8_t ptrdiff_t const uint8_t ptrdiff_t int intptr_t intptr_t int int16_t * dst
static int config_input(AVFilterLink *inlink)
const FFFilter ff_vf_dblur
int ff_filter_frame(AVFilterLink *link, AVFrame *frame)
Send a frame of data to the next filter.
int ff_filter_process_command(AVFilterContext *ctx, const char *cmd, const char *arg, char *res, int res_len, int flags)
Generic processing of user supplied commands that are set in the same way as the filter options.
Main libavfilter public API header.
#define AV_CEIL_RSHIFT(a, b)
static av_always_inline av_const unsigned av_clip_uintp2_c(int a, int p)
Clip a signed integer to an unsigned power of two range.
static __device__ float sqrtf(float a)
static __device__ float fabsf(float a)
static int filter_frame(DBEDecodeContext *s, AVFrame *frame)
@ AV_OPT_TYPE_INT
Underlying C type is int.
@ AV_OPT_TYPE_FLOAT
Underlying C type is float.
#define AVFILTER_FLAG_SUPPORT_TIMELINE_GENERIC
Some filters support a generic "enable" expression option that can be used to enable or disable a fil...
int av_frame_is_writable(AVFrame *frame)
Check if the frame data is writable.
void av_frame_free(AVFrame **frame)
Free the frame and any dynamically allocated objects in it, e.g.
int av_frame_copy_props(AVFrame *dst, const AVFrame *src)
Copy only "metadata" fields from src to dst.
void av_image_copy_plane(uint8_t *dst, int dst_linesize, const uint8_t *src, int src_linesize, int bytewidth, int height)
Copy image plane from src to dst.
static av_cold void uninit(AVBitStreamFilterContext *ctx)
#define FILTER_INPUTS(array)
#define FILTER_OUTPUTS(array)
#define FILTER_PIXFMTS_ARRAY(array)
#define AVFILTER_DEFINE_CLASS(fname)
#define NULL_IF_CONFIG_SMALL(x)
Return NULL if CONFIG_SMALL is true, otherwise the argument without modification.
static enum AVPixelFormat pix_fmts[]
static const struct @257111027162314367033347246032313251342043035002 planes[]
Memory handling functions.
int av_pix_fmt_count_planes(enum AVPixelFormat pix_fmt)
const AVPixFmtDescriptor * av_pix_fmt_desc_get(enum AVPixelFormat pix_fmt)
#define AV_PIX_FMT_GBRAP12
#define AV_PIX_FMT_YUV420P16
#define AV_PIX_FMT_GBRPF32
#define AV_PIX_FMT_YUV444P12
#define AV_PIX_FMT_YUV444P9
#define AV_PIX_FMT_YUV420P10
#define AV_PIX_FMT_YUV440P12
#define AV_PIX_FMT_GBRAP16
#define AV_PIX_FMT_YUV422P9
#define AV_PIX_FMT_YUVA444P10
#define AV_PIX_FMT_YUVA420P16
#define AV_PIX_FMT_YUV420P12
#define AV_PIX_FMT_YUVA420P10
#define AV_PIX_FMT_YUVA422P9
#define AV_PIX_FMT_YUV422P12
#define AV_PIX_FMT_GBRP10
#define AV_PIX_FMT_YUV422P10
#define AV_PIX_FMT_GRAY12
#define AV_PIX_FMT_GBRP12
#define AV_PIX_FMT_YUV420P9
#define AV_PIX_FMT_YUVA420P9
#define AV_PIX_FMT_YUVA422P10
#define AV_PIX_FMT_GRAYF32
#define AV_PIX_FMT_YUV420P14
AVPixelFormat
Pixel format.
@ AV_PIX_FMT_YUV420P
planar YUV 4:2:0, 12bpp, (1 Cr & Cb sample per 2x2 Y samples)
@ AV_PIX_FMT_YUV440P
planar YUV 4:4:0 (1 Cr & Cb sample per 1x2 Y samples)
@ AV_PIX_FMT_YUV422P
planar YUV 4:2:2, 16bpp, (1 Cr & Cb sample per 2x1 Y samples)
@ AV_PIX_FMT_GRAY8
Y , 8bpp.
@ AV_PIX_FMT_YUVA420P
planar YUV 4:2:0, 20bpp, (1 Cr & Cb sample per 2x2 Y & A samples)
@ AV_PIX_FMT_YUVJ440P
planar YUV 4:4:0 full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV440P and setting color_range
@ AV_PIX_FMT_YUV410P
planar YUV 4:1:0, 9bpp, (1 Cr & Cb sample per 4x4 Y samples)
@ AV_PIX_FMT_YUV411P
planar YUV 4:1:1, 12bpp, (1 Cr & Cb sample per 4x1 Y samples)
@ AV_PIX_FMT_YUV444P
planar YUV 4:4:4, 24bpp, (1 Cr & Cb sample per 1x1 Y samples)
@ AV_PIX_FMT_YUVA444P
planar YUV 4:4:4 32bpp, (1 Cr & Cb sample per 1x1 Y & A samples)
@ AV_PIX_FMT_YUVJ411P
planar YUV 4:1:1, 12bpp, (1 Cr & Cb sample per 4x1 Y samples) full scale (JPEG), deprecated in favor ...
@ AV_PIX_FMT_GBRAP
planar GBRA 4:4:4:4 32bpp
@ AV_PIX_FMT_YUVJ422P
planar YUV 4:2:2, 16bpp, full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV422P and setting col...
@ AV_PIX_FMT_YUVA422P
planar YUV 4:2:2 24bpp, (1 Cr & Cb sample per 2x1 Y & A samples)
@ AV_PIX_FMT_GBRP
planar GBR 4:4:4 24bpp
@ AV_PIX_FMT_YUVJ444P
planar YUV 4:4:4, 24bpp, full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV444P and setting col...
@ AV_PIX_FMT_YUVJ420P
planar YUV 4:2:0, 12bpp, full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV420P and setting col...
#define AV_PIX_FMT_YUVA422P12
#define AV_PIX_FMT_YUV422P14
#define AV_PIX_FMT_GRAY10
#define AV_PIX_FMT_GRAY14
#define AV_PIX_FMT_YUV422P16
#define AV_PIX_FMT_GRAY16
#define AV_PIX_FMT_GBRAP10
#define AV_PIX_FMT_YUVA444P16
#define AV_PIX_FMT_YUVA422P16
#define AV_PIX_FMT_GBRP16
#define AV_PIX_FMT_YUV444P14
#define AV_PIX_FMT_YUVA444P9
#define AV_PIX_FMT_GBRP14
#define AV_PIX_FMT_YUVA444P12
#define AV_PIX_FMT_GBRAPF32
#define AV_PIX_FMT_YUV444P16
#define AV_PIX_FMT_YUV444P10
Describe the class of an AVClass context structure.
void * priv
private data for use by the filter
A link between two filters.
int w
agreed upon image width
int h
agreed upon image height
AVFilterContext * dst
dest filter
int format
agreed upon media format
A filter pad used for either input or output.
This structure describes decoded (raw) audio or video data.
uint8_t * data[AV_NUM_DATA_POINTERS]
pointer to the picture/channel planes.
int linesize[AV_NUM_DATA_POINTERS]
For video, a positive or negative value, which is typically indicating the size in bytes of each pict...
Descriptor that unambiguously describes how the bits of a pixel are stored in the up to 4 data planes...
#define av_malloc_array(a, b)
static AVFormatContext * ctx
static int config_input(AVFilterLink *inlink)
static void set_params(DBlurContext *s, float angle, float r)
static int filter_frame(AVFilterLink *inlink, AVFrame *in)
static int filter_horizontally(AVFilterContext *ctx, int width, int height)
static av_cold void uninit(AVFilterContext *ctx)
static const AVOption dblur_options[]
static const AVFilterPad dblur_inputs[]
static void diriir2d(AVFilterContext *ctx, int plane)
static double b1(void *priv, double x, double y)
static double a0(void *priv, double x, double y)
static double a3(void *priv, double x, double y)
static double b0(void *priv, double x, double y)
static double a2(void *priv, double x, double y)
static double a1(void *priv, double x, double y)
const AVFilterPad ff_video_default_filterpad[1]
An AVFilterPad array whose only entry has name "default" and is of type AVMEDIA_TYPE_VIDEO.
AVFrame * ff_get_video_buffer(AVFilterLink *link, int w, int h)
Request a picture buffer with a specific set of permissions.